@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -29,51 +29,42 @@ namespace mobius
 namespace imagefile
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get URL for first segment file
-//! \param url imagefile URL
-//! \return first segment file URL
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static const std::string
-get_first_segment_url (const std::string& url)
-{
-  return url.substr (0, url.length () - 4) + ".001";
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief check if URL is an instance of imagefile solo
+//! \brief Check if file is an instance of imagefile solo
+//! \param f File object
 //! \return true/false
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 bool
-imagefile_impl_solo::is_instance (const std::string& url)
+imagefile_impl_solo::is_instance (mobius::io::file f)
 {
   bool instance = false;
 
-  mobius::io::file f (url);
-
   if (f && f.exists ())
     {
       auto reader = f.new_reader ();
-      mobius::bytearray data = reader.read (14);
-
-      instance = data == "[SEIZE HEADER]";
+      
+      if (reader)
+        {
+          mobius::bytearray data = reader.read (14);
+          instance = data == "[SEIZE HEADER]";
+        }
     }
 
   return instance;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief construct object
-//! \param url imagefile URL
+//! \brief Construct object
+//! \param f File object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-imagefile_impl_solo::imagefile_impl_solo (const std::string& url)
-  : url_ (url),
-    split_imagefile_ (get_first_segment_url (url))
+imagefile_impl_solo::imagefile_impl_solo (mobius::io::file f)
+  : file_ (f),
+    split_imagefile_ (f.new_sibling_by_extension ("001"))
 {
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create new reader for imagefile
-//! \return reader
+//! \brief Create new reader for imagefile
+//! \return Reader object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::io::reader
 imagefile_impl_solo::new_reader () const
@@ -82,8 +73,8 @@ imagefile_impl_solo::new_reader () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create new writer for imagefile
-//! \return writer
+//! \brief Create new writer for imagefile
+//! \return Writer object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::io::writer
 imagefile_impl_solo::new_writer () const
@@ -92,7 +83,7 @@ imagefile_impl_solo::new_writer () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get metadata
+//! \brief Get metadata
 //! \return imagefile metadata
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::metadata
@@ -100,12 +91,6 @@ imagefile_impl_solo::get_metadata () const
 {
   return mobius::metadata
   {
-    {
-      "url",
-      "URL",
-      "std::string",
-      get_url ()
-    },
     {
       "type",
       "type",
@@ -194,7 +179,7 @@ imagefile_impl_solo::get_metadata () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief load metadata on demand
+//! \brief Load metadata on demand
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 imagefile_impl_solo::_load_metadata () const
@@ -202,85 +187,88 @@ imagefile_impl_solo::_load_metadata () const
   if (metadata_loaded_)
     return;
 
-  // load metadata
-  mobius::io::file f (url_);
+  // Check if imagefile exists
   constexpr mobius::io::file::size_type LOG_MAX_SIZE = 1048576;  // 1 MiB
 
-  if (f.exists () && f.get_size () < LOG_MAX_SIZE)
+  if (!file_ || !file_.exists ())
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("Image file not found"));
+
+  if (file_.get_size () > LOG_MAX_SIZE)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("Image file control file too large"));
+          
+  // Load metadata
+  std::regex REGEX_SEIZE_TIME ("\\nSeize Time = *([0-9]+):([0-9]+):([0-9]+)\\n");
+  std::regex REGEX_SEIZE_DATE ("\\nSeize Date = *([0-9]+)/([0-9]+)/([0-9]+)\\n");
+  std::regex REGEX_TOTAL_SECTORS ("\\nSeize Size = ([0-9]+)\\n");
+  std::regex REGEX_MD5 ("\\nMD5: (.+)\\n");
+  std::regex REGEX_SUSPECT ("Suspect: Model: ([^.]+?)\\. Serial Number: ([^.]+?)\\..*Block size: ([0-9]+)\\..*\\. Evidence");
+  std::regex REGEX_SOLO_PRODUCT ("\\nProduct = (.+)\\n");
+  std::regex REGEX_SOLO_SERIAL ("\\nSerial #: (.+)\\n");
+  std::regex REGEX_SOLO_SOFTWARE ("\\nSoftware Version (.+)\\n");
+
+  // parse .txt file
+  auto reader = file_.new_reader ();
+  mobius::bytearray data = reader.read (file_.get_size ());
+  const std::string text = conv_charset_to_utf8 (data, "ASCII");
+  std::smatch match;
+  mobius::datetime::date acquisition_date;
+  std::string product;
+  std::string serial;
+
+  if (std::regex_search (text, match, REGEX_SUSPECT))
     {
-      std::regex REGEX_SEIZE_TIME ("\\nSeize Time = *([0-9]+):([0-9]+):([0-9]+)\\n");
-      std::regex REGEX_SEIZE_DATE ("\\nSeize Date = *([0-9]+)/([0-9]+)/([0-9]+)\\n");
-      std::regex REGEX_TOTAL_SECTORS ("\\nSeize Size = ([0-9]+)\\n");
-      std::regex REGEX_MD5 ("\\nMD5: (.+)\\n");
-      std::regex REGEX_SUSPECT ("Suspect: Model: ([^.]+?)\\. Serial Number: ([^.]+?)\\..*Block size: ([0-9]+)\\..*\\. Evidence");
-      std::regex REGEX_SOLO_PRODUCT ("\\nProduct = (.+)\\n");
-      std::regex REGEX_SOLO_SERIAL ("\\nSerial #: (.+)\\n");
-      std::regex REGEX_SOLO_SOFTWARE ("\\nSoftware Version (.+)\\n");
-
-      // parse .txt file
-      auto reader = f.new_reader ();
-      mobius::bytearray data = reader.read (f.get_size ());
-      const std::string text = conv_charset_to_utf8 (data, "ASCII");
-      std::smatch match;
-      mobius::datetime::date acquisition_date;
-      std::string product;
-      std::string serial;
-
-      if (std::regex_search (text, match, REGEX_SUSPECT))
-        {
-          drive_model_ = match[1].str ();
-          drive_serial_number_ = match[2].str ();
-          sector_size_ = stoi (match[3].str ());
-        }
-
-      if (std::regex_search (text, match, REGEX_SEIZE_DATE))
-        {
-          acquisition_date = mobius::datetime::date (
-                                stoi (match[3].str ()),
-                                stoi (match[1].str ()),
-                                stoi (match[2].str ()));
-
-        }
-     
-      if (std::regex_search (text, match, REGEX_SEIZE_TIME))
-        {
-          mobius::datetime::time time = mobius::datetime::time (
-                                          stoi (match[1].str ()),
-                                          stoi (match[2].str ()),
-                                          stoi (match[3].str ()));
-
-          if (acquisition_date)
-            acquisition_time_ = mobius::datetime::datetime (acquisition_date, time);
-        }
-
-      if (std::regex_search (text, match, REGEX_TOTAL_SECTORS))
-        {
-          sectors_ = stoll (match[1].str ());
-          size_ = sectors_ * sector_size_;
-        }
-
-      if (std::regex_search (text, match, REGEX_MD5))
-        {
-          std::string value = mobius::string::remove_char (match[1].str (), ' ');
-          hash_md5_ = mobius::string::tolower (value);
-        }
-
-      if (std::regex_search (text, match, REGEX_SOLO_PRODUCT))
-        product = match[1].str ();
-
-      if (std::regex_search (text, match, REGEX_SOLO_SERIAL) && !product.empty ())
-        acquisition_platform_ = product + " (s/n: " + match[1].str () + ')';
-
-      if (std::regex_search (text, match, REGEX_SOLO_SOFTWARE))
-        acquisition_tool_ = "Solo software v" + match[1].str ();
-
-      segments_ = split_imagefile_.get_segments ();
-      segment_size_ = split_imagefile_.get_segment_size ();
-      acquisition_user_ = f.get_user_name ();
-
-      normalize_drive_info (drive_vendor_, drive_model_, drive_serial_number_);
+      drive_model_ = match[1].str ();
+      drive_serial_number_ = match[2].str ();
+      sector_size_ = stoi (match[3].str ());
     }
 
+  if (std::regex_search (text, match, REGEX_SEIZE_DATE))
+    {
+      acquisition_date = mobius::datetime::date (
+                            stoi (match[3].str ()),
+                            stoi (match[1].str ()),
+                            stoi (match[2].str ()));
+
+    }
+     
+  if (std::regex_search (text, match, REGEX_SEIZE_TIME))
+    {
+      mobius::datetime::time time = mobius::datetime::time (
+                                      stoi (match[1].str ()),
+                                      stoi (match[2].str ()),
+                                      stoi (match[3].str ()));
+
+      if (acquisition_date)
+        acquisition_time_ = mobius::datetime::datetime (acquisition_date, time);
+    }
+
+  if (std::regex_search (text, match, REGEX_TOTAL_SECTORS))
+    {
+      sectors_ = stoll (match[1].str ());
+      size_ = sectors_ * sector_size_;
+    }
+
+  if (std::regex_search (text, match, REGEX_MD5))
+    {
+      std::string value = mobius::string::remove_char (match[1].str (), ' ');
+      hash_md5_ = mobius::string::tolower (value);
+    }
+
+  if (std::regex_search (text, match, REGEX_SOLO_PRODUCT))
+    product = match[1].str ();
+
+  if (std::regex_search (text, match, REGEX_SOLO_SERIAL) && !product.empty ())
+    acquisition_platform_ = product + " (s/n: " + match[1].str () + ')';
+
+  if (std::regex_search (text, match, REGEX_SOLO_SOFTWARE))
+    acquisition_tool_ = "Solo software v" + match[1].str ();
+
+  segments_ = split_imagefile_.get_segments ();
+  segment_size_ = split_imagefile_.get_segment_size ();
+  acquisition_user_ = file_.get_user_name ();
+
+  normalize_drive_info (drive_vendor_, drive_model_, drive_serial_number_);
+ 
   // set metadata loaded
   metadata_loaded_ = true;
 }
