@@ -21,6 +21,7 @@
 #include <mobius/bytearray.h>
 #include <mobius/encoder/data_encoder.h>
 #include <cstdint>
+#include <stdexcept>
 
 namespace mobius
 {
@@ -40,7 +41,7 @@ static constexpr std::uint32_t VERSION = 1;
 //! \param encoder Encoder object
 //! \param d Data object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-void
+static void
 _serialize_data (mobius::encoder::data_encoder& encoder, const data& data)
 {
   switch (data.get_type ())
@@ -127,6 +128,18 @@ _serialize_data (mobius::encoder::data_encoder& encoder, const data& data)
     }
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Serialize data object
+//! \param data Data object
+//! \return Encoded data
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static void
+_serialize (mobius::encoder::data_encoder& encoder, const data& data)
+{
+  encoder.encode_uint32_le (VERSION);
+  _serialize_data (encoder, data);
+}
+
 } // namespace
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -146,5 +159,17 @@ serialize (const data& data)
   return out;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Serialize data object
+//! \param data Data object
+//! \return Encoded data
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+serialize (mobius::io::writer writer, const data& data)
+{
+  mobius::encoder::data_encoder encoder (writer);
+  _serialize (encoder, data);
+}
+
 } // namespace pod
 } // namespace mobius
