@@ -29,19 +29,25 @@ PyObject *
 func_pbkdf2_hmac (PyObject *, PyObject *args)
 {
   // parse input args
-  const std::uint8_t *arg_key_buffer;
-  Py_ssize_t arg_key_size;
-  const std::uint8_t *arg_salt_buffer;
-  Py_ssize_t arg_salt_size;
-  unsigned long arg_iterations = 0;
-  unsigned int arg_dklen = 0;
-  const char *arg_hash_id = "sha1";
+  std::string arg_hash_id;
+  mobius::bytearray arg_key;
+  mobius::bytearray arg_salt;
+  std::uint32_t arg_iterations;
+  std::uint16_t arg_dklen;
 
-  if (!PyArg_ParseTuple (args, "ss#s#LI", &arg_hash_id, &arg_key_buffer, &arg_key_size, &arg_salt_buffer, &arg_salt_size, &arg_iterations, &arg_dklen))
-    return nullptr;
-
-  mobius::bytearray arg_key (arg_key_buffer, arg_key_size);
-  mobius::bytearray arg_salt (arg_salt_buffer, arg_salt_size);
+  try
+    {
+      arg_hash_id = mobius::py::get_arg_as_std_string (args, 0);
+      arg_key = mobius::py::get_arg_as_bytearray (args, 1);
+      arg_salt = mobius::py::get_arg_as_bytearray (args, 2);
+      arg_iterations = mobius::py::get_arg_as_uint32_t (args, 3);
+      arg_dklen = mobius::py::get_arg_as_uint16_t (args, 4);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -64,6 +70,5 @@ func_pbkdf2_hmac (PyObject *, PyObject *args)
       PyErr_SetString (PyExc_IOError, e.what ());
     }
 
-  // return value
   return ret;
 }
