@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -21,7 +21,11 @@
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include <pymobius.h>
+#include <pygil.h>
 #include "reader.h"
+#include <mobius/exception.inc>
+#include <mobius/io/bytearray_io.h>
+#include <stdexcept>
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Check if object type is <i>reader</i>
@@ -53,8 +57,7 @@ PyMobius_IO_Reader_from_cpp (mobius::io::reader obj)
     }
   else
     {
-      Py_INCREF (Py_None);
-      ret = Py_None;
+      ret = mobius::py::pynone ();
     }
 
   return ret;
@@ -68,7 +71,27 @@ PyMobius_IO_Reader_from_cpp (mobius::io::reader obj)
 mobius::io::reader
 PyMobius_IO_Reader_as_cpp (PyObject *pyobj)
 {
-  return * (reinterpret_cast <io_reader_o *>(pyobj)->obj);
+  mobius::io::reader reader;
+
+  if (PyMobius_IO_Reader_Check (pyobj))
+    reader = * (reinterpret_cast <io_reader_o *>(pyobj)->obj);
+  
+  else if (mobius::py::pybytes_check (pyobj))
+    {
+      auto bytearray = mobius::py::pybytes_as_bytearray (pyobj);
+      reader = mobius::io::new_bytearray_reader (bytearray);
+    }
+
+  else if (mobius::py::pystring_check (pyobj))
+    {
+      mobius::bytearray data (mobius::py::pystring_as_std_string (pyobj));
+      reader = mobius::io::new_bytearray_reader (data);
+    }
+
+  else
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("argument must be either reader or bytes"));
+
+  return reader;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -121,7 +144,7 @@ tp_getter_size (io_reader_o *self)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_size ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_size ());
     }
   catch (const std::exception& e)
     {
@@ -168,30 +191,28 @@ tp_f_read (io_reader_o *self, PyObject *args)
   // parse input args
   try
     {
-      arg_size = self->obj->get_size () - self->obj->tell ();
-
-      if (!PyArg_ParseTuple (args, "|K", &arg_size))
-        return nullptr;
+      arg_size = mobius::py::get_arg_as_uint64_t (
+                   args,
+                   0,
+                   self->obj->get_size () - self->obj->tell ()
+                 );
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
     }
 
   // execute C++ code
   PyObject *ret = nullptr;
-  PyThreadState *_save = PyEval_SaveThread ();
 
   try
     {
-      auto data = self->obj->read (arg_size);
-
-      PyEval_RestoreThread (_save);
+      auto data = mobius::py::GIL () (self->obj->read (arg_size));
       ret = mobius::py::pybytes_from_bytearray (data);
     }
   catch (const std::runtime_error& e)
     {
-      PyEval_RestoreThread(_save);
       PyErr_SetString (PyExc_IOError, e.what ());
     }
 
@@ -208,7 +229,7 @@ tp_f_tell (io_reader_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->tell ());
+      ret = mobius::py::pylong_from_std_int64_t (self->obj->tell ());
     }
   catch (const std::exception& e)
     {
@@ -226,11 +247,20 @@ tp_f_seek (io_reader_o *self, PyObject *args)
 {
   // parse input args
   std::int64_t arg_offset;
-  int arg_whence = 0;
+  std::uint32_t arg_whence = 0;
 
-  if (!PyArg_ParseTuple (args, "L|i", &arg_offset, &arg_whence))
-    return nullptr;
+  try
+    {
+      arg_offset = mobius::py::get_arg_as_int64_t (args, 0);
+      arg_whence = mobius::py::get_arg_as_uint32_t (args, 1, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
+  // convert 'whence' argument
   mobius::io::reader::whence_type w;
 
   if (arg_whence == 0)
@@ -244,7 +274,7 @@ tp_f_seek (io_reader_o *self, PyObject *args)
 
   else
     {
-      PyErr_SetString (PyExc_ValueError, "Invalid 'whence' argument");
+      PyErr_SetString (PyExc_ValueError, "invalid 'whence' argument");
       return nullptr;
     }
 
@@ -254,12 +284,11 @@ tp_f_seek (io_reader_o *self, PyObject *args)
   try
     {
       self->obj->seek (arg_offset, w);
-      Py_INCREF (Py_None);
-      ret = Py_None;
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      PyErr_SetString (PyExc_IOError, e.what ());
     }
 
   return ret;
@@ -276,8 +305,7 @@ tp_f_rewind (io_reader_o *self, PyObject *)
   try
     {
       self->obj->rewind ();
-      Py_INCREF (Py_None);
-      ret = Py_None;
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
@@ -294,19 +322,25 @@ static PyObject *
 tp_f_skip (io_reader_o *self, PyObject *args)
 {
   // parse input args
-  std::int64_t arg_size;
+  std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "L", &arg_size))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
-  // execute C++ function
+  // execute C++ code
   PyObject *ret = nullptr;
 
   try
     {
       self->obj->skip (arg_size);
-      Py_INCREF (Py_None);
-      ret = Py_None;
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
@@ -317,7 +351,27 @@ tp_f_skip (io_reader_o *self, PyObject *args)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief methods structure
+//! \brief <b>get_block_size</b> method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_block_size (io_reader_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->tell ());
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Methods structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyMethodDef tp_methods[] =
 {
@@ -325,31 +379,37 @@ static PyMethodDef tp_methods[] =
     (char *) "read",
     (PyCFunction) tp_f_read,
     METH_VARARGS,
-    "read bytes from reader"
+    "Read bytes from reader"
   },
   {
     (char *) "tell",
     (PyCFunction) tp_f_tell,
     METH_VARARGS,
-    "get current read position"
+    "Get current read position"
   },
   {
     (char *) "seek",
     (PyCFunction) tp_f_seek,
     METH_VARARGS,
-    "set current read position"
+    "Set current read position"
   },
   {
     (char *) "rewind",
     (PyCFunction) tp_f_rewind,
     METH_VARARGS,
-    "set current read position to the beginning of data"
+    "Set current read position to the beginning of data"
   },
   {
     (char *) "skip",
     (PyCFunction) tp_f_skip,
     METH_VARARGS,
-    "set read position n bytes ahead"
+    "Set read position n bytes ahead"
+  },
+  {
+    (char *) "get_block_size",
+    (PyCFunction) tp_f_get_block_size,
+    METH_VARARGS,
+    "Get optimum block size"
   },
   {NULL, NULL, 0, NULL} // sentinel
 };
