@@ -31,7 +31,18 @@
 #include "database/transaction.h"
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create <i>Case</i> Python object from C++ object
+//! \brief Check if object is an instance of <i>mobius.model.case</i>
+//! \param pyobj Python object
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+PyMobius_Model_Case_Check (PyObject *pyobj)
+{
+  return PyObject_IsInstance (pyobj, (PyObject *) &model_case_t);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create <i>Case</i> Python object from C++ object
 //! \param obj C++ object
 //! \return new Case object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -46,6 +57,20 @@ PyMobius_Model_Case_from_cpp (mobius::model::Case obj)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create <i>case</i> C++ object from Python object
+//! \param value Python object
+//! \return reader object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::model::Case
+PyMobius_Model_Case_as_cpp (PyObject *value)
+{
+  if (!PyMobius_Model_Case_Check (value))
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("object must be an instance of mobius.model.case"));
+
+  return * (reinterpret_cast <model_case_o *>(value)->obj);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief <i>root_item</i> attribute getter
 //! \param self object
@@ -80,7 +105,7 @@ tp_getter_uid (model_case_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_uid ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_uid ());
     }
   catch (const std::exception& e)
     {
@@ -122,10 +147,17 @@ static PyObject *
 tp_f_get_path (model_case_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_rpath;
+  std::string arg_rpath;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_rpath))
-    return nullptr;
+  try
+    {
+      arg_rpath = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -153,10 +185,17 @@ static PyObject *
 tp_f_create_path (model_case_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_rpath;
+  std::string arg_rpath;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_rpath))
-    return nullptr;
+  try
+    {
+      arg_rpath = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -234,10 +273,17 @@ static PyObject *
 tp_f_get_item_by_uid (model_case_o *self, PyObject *args)
 {
   // parse input args
-  std::int64_t arg_uid;
+  std::uint64_t arg_uid;
 
-  if (!PyArg_ParseTuple (args, "L", &arg_uid))
-    return nullptr;
+  try
+    {
+      arg_uid = mobius::py::get_arg_as_int64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -265,11 +311,19 @@ static PyObject *
 tp_f_new_application (model_case_o *self, PyObject *args)
 {
   // parse input args
-  const char *arg_id = nullptr;
-  const char *arg_name = nullptr;
+  std::string arg_id;
+  std::string arg_name;
 
-  if (!PyArg_ParseTuple (args, "ss", &arg_id, &arg_name))
-    return nullptr;
+  try
+    {
+      arg_id = mobius::py::get_arg_as_std_string (args, 0);
+      arg_name = mobius::py::get_arg_as_std_string (args, 1);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -283,7 +337,6 @@ tp_f_new_application (model_case_o *self, PyObject *args)
       PyErr_SetString (PyExc_Exception, e.what ());
     }
 
-  // return value
   return ret;
 }
 
